package com.displaynone.acss.ui.profile

import android.util.Log
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import androidx.paging.Pager
import androidx.paging.PagingConfig
import androidx.paging.cachedIn
import com.displaynone.acss.components.auth.models.user.UserServiceST
import com.displaynone.acss.components.acs.models.visit.VisitListPagingSource
import com.displaynone.acss.components.acs.models.visit.VisitServiceST
import com.displaynone.acss.components.auth.models.user.repository.dto.UserDTO
import io.ktor.util.reflect.instanceOf
import kotlinx.coroutines.channels.BufferOverflow
import kotlinx.coroutines.channels.Channel
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.asStateFlow
import kotlinx.coroutines.flow.receiveAsFlow
import kotlinx.coroutines.launch

class ProfileViewModel(): ViewModel() {
    private val _action = Channel<Action>(
        capacity = Channel.BUFFERED,
        onBufferOverflow = BufferOverflow.DROP_OLDEST,
    )
    private var login: String = ""
    fun setLogin(login1: String){
        login = login1
    }
    val action = _action.receiveAsFlow()
    val _state = MutableStateFlow<State>(State.Loading)
    val state = _state.asStateFlow()

    val visitListState = Pager(
        config = PagingConfig(pageSize = 20,
            enablePlaceholders = false,
            maxSize = 100
        )
    ) {
        VisitListPagingSource(VisitServiceST.getInstance()::getMyLastVisits)
    }.flow
        .cachedIn(viewModelScope)

    val visitListStateFromLogin = Pager(
        config = PagingConfig(pageSize = 20, enablePlaceholders = false, maxSize = 100)
    ) {
        VisitListPagingSource { pageNum, pageSize ->
            VisitServiceST.getInstance().getLastVisitsByLogin(pageNum,pageSize, login)
        }
    }.flow.cachedIn(viewModelScope)

    fun logout(){
        UserServiceST.getInstance().logout()
    }
    fun getInfo(){
        viewModelScope.launch {
            UserServiceST.getInstance().getInfo().fold(
                onSuccess = { data ->
                    _state.emit(State.Show(data))
                    UserServiceST.getInstance().saveUserDTO(data)
                    Log.d("Pvm", data.login)
                },
                onFailure = { error ->
                    error.message?.let { error(it) } //FIXME()
                    Log.e("ProfileViewModel", error.message.toString())
                }
            )
        }
    }
    fun changeRights(login: String, isACSBlocked: Boolean){
        viewModelScope.launch {
            UserServiceST.getInstance().changeRights(login, isACSBlocked).fold(
                onSuccess = {
                    Log.d("ProfileViewModel", "changed rights")
                    _state.emit(State.Change(isACSBlocked))
                },
                onFailure = { error ->
                    Log.d("ProfileViewModel", "failed to change rights ${error.message}")
                }
            )
        }
    }
    fun openAuth(){
        viewModelScope.launch {
            _action.send(Action.GoToAuth)
        }
    }
    fun openScan(){
        viewModelScope.launch {
            _action.send(Action.GoToScan)
        }
    }

    sealed interface State {
        data object Loading : State
        data class Show(
            val item: UserDTO
        ) : State
        data class Change(
            val item: Boolean
        ) : State
    }
    sealed interface Action {
        data object GoToAuth: Action
        data object GoToScan: Action
    }
}